#define com_maroontress_libcgi_db_Box_IMPORT
#define com_maroontress_libcgi_db_BoxKit_IMPORT

#include <stdlib.h>
#include <stdio.h>
#include <com/maroontress/libcgi/db/Box.h>
#include <com/maroontress/libcgi/db/BoxKit.h>
#include "Platform.h"

static void
setKeyName(void *key, size_t keyLen, int num, const char *subName)
{
    snprintf(key, keyLen, "platform%d.%s", num, subName);
}

int
Platform_getStringWithBox(struct Box *box, int k, const char *name,
			  char **value)
{
    char fullName[64];

    setKeyName(fullName, sizeof(fullName), k, name);
    return Box_getString(box, fullName, value);
}

int
Platform_getIntWithBox(struct Box *box, int k, const char *name, int *value)
{
    char fullName[64];

    setKeyName(fullName, sizeof(fullName), k, name);
    return Box_getInt(box, fullName, value);
}

int
Platform_setStringWithBoxKit(struct BoxKit *kit, int k, const char *name,
			     const char *value)
{
    char fullName[64];
    
    setKeyName(fullName, sizeof(fullName), k, name);
    return BoxKit_setString(kit, fullName, value);
}

int
Platform_setIntWithBoxKit(struct BoxKit *kit, int k, const char *name,
			  int value)
{
    char fullName[64];
    
    setKeyName(fullName, sizeof(fullName), k, name);
    return BoxKit_setInt(kit, fullName, value);
}

static int
createStringWithBoxKit(struct BoxKit *kit, int k, const char *name,
		    char **value)
{
    char fullName[64];

    setKeyName(fullName, sizeof(fullName), k, name);
    return BoxKit_createString(kit, fullName, value);
}

static int
initializeWithBoxKit(struct Platform *p, struct BoxKit *kit, unsigned int k)
{
    if (createStringWithBoxKit(kit, k, "id", &p->id) < 0
	|| createStringWithBoxKit(kit, k, "host", &p->host) < 0
	|| createStringWithBoxKit(kit, k, "command", &p->command) < 0
	|| createStringWithBoxKit(kit, k, "charset", &p->charset) < 0) {
	return -1;
    }
    return 0;
}

struct Platform *
Platform_createWithBoxKit(struct BoxKit *kit, unsigned int k)
{
    struct Platform *p;

    if ((p = (struct Platform *)malloc(sizeof(*p))) == NULL)
	return NULL;
    p->id = NULL;
    p->host = NULL;
    p->command = NULL;
    p->charset = NULL;
    if (initializeWithBoxKit(p, kit, k) < 0) {
	Platform_free(p);
	return NULL;
    }
    return p;
}

void
Platform_free(struct Platform *p)
{
    if (p == NULL)
	return;
    free(p->id);
    free(p->host);
    free(p->command);
    free(p->charset);
    free(p);
}
